package my.com.connector.ws.client.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import my.com.connector.ws.client.data.EventLog;


/**
 * Implements the {@link EventLogDao}.
 * 
 * @author chinyeh
 *
 */
public class BasicEventLogDao implements EventLogDao {

	private static final String TABLE_NAME = "EVENT_LOG";
	
	private Connection connection;
	
	/**
	 * @param connection an reference of the established database connection
	 */
	public BasicEventLogDao(Connection connection) {
		this.connection = connection;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public EventLog persist(EventLog eventLog) {
		PreparedStatement statement = null;
		ResultSet resultSet = null;
		
		StringBuilder sql = new StringBuilder()
			.append("insert into " + TABLE_NAME)
			.append(" (request_url, request_body, mall_ind")
			.append(" , trx_type, client_trx_id, status")
			.append(" , error_msg, callback_url, last_updated")
			.append(" , http_method)")
			.append(" values(?, ?, ?")
			.append(" , ?, ?, ?")
			.append(" , ?, ?, current timestamp")
			.append(" , ?)");
		
		try {
			statement = connection.prepareStatement(sql.toString(), 
					Statement.RETURN_GENERATED_KEYS);
			
			int count = 1;
			statement.setString(count++, eventLog.getRequestUrl());
			statement.setString(count++, eventLog.getRequestBody());
			statement.setString(count++, eventLog.getMallIndicator());
			
			statement.setString(count++, eventLog.getTrxType());
			statement.setString(count++, eventLog.getClientTrxId());
			statement.setString(count++, eventLog.getStatus());
			
			statement.setString(count++, eventLog.getErrorMsg());
			statement.setString(count++, eventLog.getCallbackUrl());
			statement.setString(count++, eventLog.getHttpMethod());
			
			statement.executeUpdate();
			
			resultSet = statement.getGeneratedKeys();
			
			if (resultSet.next()) {
				eventLog.setId(resultSet.getInt(1));
			} else {
				throw new IllegalStateException(
						"No ID was generated for " + TABLE_NAME);
			}
			
		} catch (SQLException ex) {
			throw new RuntimeException(
					"Error in executing sql " + sql.toString(), ex);
		} finally {
			release(resultSet, statement);
		}
		
		return eventLog;
	}

	/**
	 * {@inheritDoc}
	 */
	public int updateEventById(int id, String eventStatus, String errorMsg) {
		int affectedRow = 0;
		
		PreparedStatement statement = null;
		StringBuilder sql = new StringBuilder()
			.append("update " + TABLE_NAME)
			.append(" set status = ?")
			.append(" , error_msg = ?")
			.append(" , last_updated = current timestamp")
			.append(" where id = ?");
		
		try {
			statement = connection.prepareStatement(sql.toString());
			
			int count = 1;
			statement.setString(count++, eventStatus);
			statement.setString(count++, errorMsg);
			statement.setInt(count++, id);
			
			affectedRow = statement.executeUpdate();
			
		} catch (SQLException ex) {
			throw new RuntimeException(
					"Error in executing sql " + sql.toString(), ex);
		} finally {
			release(null, statement);
		}
		
		return affectedRow;
	}
	
	/**
	 * @param resultSet ResultSet object
	 * @param statement PreparedStatement object
	 */
	private void release(ResultSet resultSet, PreparedStatement statement) {
		try {
			if (resultSet != null) {
				resultSet.close();
			}
			
			if (statement != null) {
				statement.close();
			}
		} catch (SQLException ex) {
			throw new RuntimeException(
					"Error in releasing sql resources", ex);
		}
	}

}
